﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Threading;
using System.Text;
using System.Windows.Forms;
using RS232Port;

namespace NavigationController {
	public partial class NavigationControllerMainForm : Form {

		NavigationStrategy _strategy;

		bool _silent = true;
		public bool Silent {
			get { return _silent; }
			set {
				_silent = value;
				if (_silent) {
					_strategy.updateUiEvent -= updateUi;
					_strategy.setSilentModeEvent -= setSilentMode;
				} else {
					_strategy.updateUiEvent += updateUi;
					_strategy.setSilentModeEvent += setSilentMode;
				}
			}
		}



		public NavigationControllerMainForm() {
			InitializeComponent();
			_strategy = new NavigationStrategy("COM5", 16.13333);
			initializeObject();
		}

		public NavigationControllerMainForm(ref NavigationStrategy strategy) {
			InitializeComponent();
			_strategy = strategy;
			initializeObject();
			onInitialize(); //call because strategy object was initialized non-graphically
		}

		private void initializeObject() {
			//foreach (PlatformCommand c in PlatformCommands.cmdArray)
			//    javelinCommandListBox.Items.Add(c.commandName);
			_strategy.updateStatusEvent += onInitialize;
			toolStripStatusLabel2.Text = _strategy.Status;
		}

		//called by delegate
		private void setSilentMode() {
			if (Silent != _strategy.Silent)
				Silent = _strategy.Silent;
		}

		private void onInitialize() {
			if (!_silent) {
				toolStripStatusLabel1.Text = _strategy.Data.Status;
				if (_strategy.Data.Status == "init")
					toolStripStatusLabel1.BackColor = Color.Green;
				else
					toolStripStatusLabel1.BackColor = Color.Red;
			}
		}

		private void launchMakeItCToolStripMenuItem_Click(object sender, EventArgs e) {
			System.Diagnostics.Process makeItC = new System.Diagnostics.Process();
			makeItC.StartInfo.FileName = "C:\\Program Files\\Coridium\\bin\\MakeItC.exe";
			makeItC.StartInfo.WorkingDirectory = "C:\\Program Files\\Coridium\\bin\\";
			makeItC.Start();
			System.Diagnostics.Process npp = new System.Diagnostics.Process();
			npp.StartInfo.FileName = "C:\\Program Files\\Notepad++\\notepad++.exe";
			npp.Start();
		}

		private void coridiumHideButton_Click(object sender, EventArgs e) {
			this.Hide();
		}


		private void openCom5Button_Click(object sender, EventArgs e) {
			toolStripStatusLabel2.Text = _strategy.openPort();
			toolStripStatusLabel1.Text = _strategy.Data.Status;
			if (_strategy.Data.Status == "init")
				toolStripStatusLabel1.BackColor = Color.Green;
		}

		private void closeCom5Button_Click(object sender, EventArgs e) {
			toolStripStatusLabel2.Text = _strategy.closePort();
			toolStripStatusLabel1.BackColor = Color.Red;
			toolStripStatusLabel1.Text = "not initialized";
		}

		private void resetCoridiumcautionToolStripMenuItem_Click(object sender, EventArgs e) {
			_strategy.resetNavigationController();
		}


		private void navigationControllerSerialPortToolStripMenuItem_Click(object sender, EventArgs e) {
			_strategy.showSerialPort();
		}

		private void silentCheckBox_CheckedChanged(object sender, EventArgs e) {
			_strategy.setSilentMode(silentCheckBox.Checked);
			Silent = silentCheckBox.Checked;
			onInitialize();
		}

		private void updateUi() {
			debugTextBox.Text = _strategy.Data.NavigationBuffer;
			xAccTextBox.Text = Math.Round(_strategy.Data.XAccAverage, 2).ToString();
			yAccTextBox.Text = Math.Round(_strategy.Data.YAccAverage, 2).ToString();
			zAccTextBox.Text = Math.Round(_strategy.Data.ZAccAverage, 2).ToString();
			leftIrRawTextBox.Text = Math.Round(_strategy.Data.LeftIrList.Average(), 2).ToString();
			leftIrTextBox.Text = Math.Round(_strategy.Data.LeftIrAverage, 2).ToString();
			rightIrRawTextBox.Text = Math.Round(_strategy.Data.RightIrList.Average(), 2).ToString();
			rightIrTextBox.Text = Math.Round(_strategy.Data.RightIrAverage, 2).ToString();
			compassTextBox.Text = Math.Round(_strategy.Data.CompassHeading, 2).ToString();
			systemVoltageTextBox.Text = Math.Round(_strategy.Data.SystemBatteryAverage, 2).ToString();
			platformVoltageTextBox.Text = Math.Round(_strategy.Data.PlatformBatteryAverage, 2).ToString();
			dataUpdateRateTextBox.Text = _strategy.WatchdogTimer.ElapsedMilliseconds.ToString();
		}

		private void clearDebugWindowButton_Click(object sender, EventArgs e) {
			debugTextBox.Clear();
		}

		private void ledOnLedButton_Click(object sender, EventArgs e) {
			_strategy.writeToNavigationController(NavigationCommands.cmdArray
				[(int)NavigationCommands.commands.systemControllerUp]);
		}

		private void flashLedButton_Click(object sender, EventArgs e) {
			_strategy.writeToNavigationController(NavigationCommands.cmdArray
				[(int)NavigationCommands.commands.errorFlashLed]);
		}

		private void turnOffLedButton_Click(object sender, EventArgs e) {
			_strategy.writeToNavigationController(NavigationCommands.cmdArray
				[(int)NavigationCommands.commands.resetEncoderLedOff]);
		}
	}
}
