﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;


namespace PlatformController {
    public partial class PlatformControllerMainForm : Form {

		//this form is just a shell for the Platform Strategy class
		PlatformStrategy _strategy;
		public PlatformStrategy Strategy {
			get { return _strategy; }
		}

        bool _silent = true;
		public bool Silent {
			get { return _silent; }
			set {
				_silent = value;
				if (_silent) {
					_strategy.updateUiEvent -= updateUi;
					_strategy.setSilentModeEvent -= setSilentMode;
				} else {
					_strategy.updateUiEvent += updateUi;
					_strategy.setSilentModeEvent += setSilentMode;
				}
			}
		}

		string _status = "ok";
		public string Status {
			get { return _status; }
		}

		//default constructor
        public PlatformControllerMainForm() {
            InitializeComponent();
			_strategy = new PlatformStrategy("COM2", "COM1", "COM3", 194.49, 0.01, 0.02, 0.02);
			initializeObject();
        }

		//shell constructor, the strategy and data objects are passed in as references
		public PlatformControllerMainForm(ref PlatformStrategy strategy) {
			InitializeComponent();
			_strategy = strategy;
			initializeObject();
			onInitialize(); //call because strategy object was initialized non-graphically
		}

		private void initializeObject() {
			_strategy.updateStatusEvent += onInitialize;
			toolStripStatusLabel2.Text = _strategy.Status;
		}

		private void onInitialize() {
			if (!_silent) {
				toolStripStatusLabel1.Text = _strategy.Data.Status;
				if (_strategy.Data.Status == "1")
					toolStripStatusLabel1.BackColor = Color.Green;
				else
					toolStripStatusLabel1.BackColor = Color.Red;
			}
		}

		//called by delegate
		private void setSilentMode() {
			if (Silent != _strategy.Silent)
				Silent = _strategy.Silent;
		}

        private void launchJavelinIDEToolStripMenuItem_Click(object sender, EventArgs e) {
            System.Diagnostics.Process javelinIDE = new System.Diagnostics.Process();
            javelinIDE.StartInfo.FileName = "C:\\Program Files\\Parallax Inc\\Javelin Stamp IDE\\Javelin Stamp IDE.exe";
            javelinIDE.StartInfo.WorkingDirectory = "C:\\Program Files\\Parallax Inc\\Javelin Stamp IDE";
            javelinIDE.Start();
        }

        private void javelinHideButton_Click(object sender, EventArgs e) {
            this.Hide();
        }

		private void updateUi() {
			debugTextBox.Text = _strategy.Data.PlatformBuffer;
			leftBumperTextBox.Text = _strategy.Data.LeftBumper.ToString();
			rightBumperTextBox.Text = _strategy.Data.RightBumper.ToString();
			leftSonarTextBox.Text = Math.Round(_strategy.Data.LeftSonar, 2).ToString();
			centerSonarTextBox.Text = Math.Round(_strategy.Data.CenterSonar, 2).ToString();
			rightSonarTextBox.Text = Math.Round(_strategy.Data.RightSonar, 2).ToString();
			throttlePercentTextBox.Text = _strategy.Data.Throttle.ToString();
			steeringPercentTextBox.Text = _strategy.Data.Steering.ToString();
			reversingTextBox.Text = _strategy.Data.Reversing.ToString();
			autoReversingTextBox.Text = _strategy.Data.AutoReversing.ToString();
			manualTextBox.Text = _strategy.Data.ManualOverride.ToString();
			encoderCountsTextBox.Text = _strategy.Data.EncoderCounts.ToString();
			countsPerSecondTextBox.Text = Math.Round(_strategy.Data.CountsPerSecond, 2).ToString();
			throttleSetpointTextBox.Text = _strategy.Data.ThrottleSetpoint.ToString();
			manulThrottleTextBox.Text = _strategy.ManualThrottle.ToString();
			throttleSetpointErrorTextBox.Text = Math.Round((double)_strategy.Data.ThrottleSetpoint
				- _strategy.Data.CountsPerSecond, 2).ToString();
			dataUpdateRateTextBox.Text = _strategy.WatchdogTimer.ElapsedMilliseconds.ToString();
			iRInstructionTextBox.Text = _strategy.Data.IrInstruction.ToString();
		}

		private void openCom2Button_Click(object sender, EventArgs e) {
			toolStripStatusLabel2.Text = _strategy.openPort();
			toolStripStatusLabel1.Text = _strategy.Data.Status;
			if (_strategy.Data.Status == "1")
				toolStripStatusLabel1.BackColor = Color.Green;
		}

        private void closeCom2Button_Click(object sender, EventArgs e) {
			toolStripStatusLabel2.Text = _strategy.closePort();
			toolStripStatusLabel1.BackColor = Color.Red;
			toolStripStatusLabel1.Text = "not initialized";
        }

        private void steeringMinusButton_Click(object sender, EventArgs e) {
			_strategy.steeringOut(--_strategy.ManualSteering);
        }

        private void steeringPlusButton_Click(object sender, EventArgs e) {
			_strategy.steeringOut(++_strategy.ManualSteering);
        }

        private void throttleMinusButton_Click(object sender, EventArgs e) {
			--_strategy.Data.ThrottleSetpoint;
        }

        private void throttlePlusButton_Click(object sender, EventArgs e) {
			++_strategy.Data.ThrottleSetpoint;
        }

        private void clearDebugWindowButton_Click(object sender, EventArgs e) {
			debugTextBox.Clear();
        }

        private void debugTextBox_KeyDown(object sender, KeyEventArgs e) {
            switch (e.KeyCode.ToString()) {
                case "Down":
					--_strategy.Data.ThrottleSetpoint;
                    break;
                case "Up":
					++_strategy.Data.ThrottleSetpoint;
                    break;
                case "Right":
					_strategy.steeringOut(--_strategy.ManualSteering);
                    break;
                case "Left":
					_strategy.steeringOut(++_strategy.ManualSteering);
                    break;
                default:
                    break;
            }
        }

        private void resetJavelinStampcautionToolStripMenuItem_Click(object sender, EventArgs e) {
			_strategy.resetPlatformController();
        }

        private void platformControllerSerialPortToolStripMenuItem_Click(object sender, EventArgs e) {
			_strategy.showSerialPort();
        }

        private void silentCheckBox_CheckedChanged(object sender, EventArgs e) {
			_strategy.setSilentMode(silentCheckBox.Checked);
			Silent = silentCheckBox.Checked;
			onInitialize();
        }

		private void throttleSetpointSetButton_Click(object sender, EventArgs e) {
			try {
				_strategy.Data.ThrottleSetpoint = int.Parse(throttleSetpointTextBox.Text);
			} catch (Exception ex) {
				// post a non numeric message
				toolStripStatusLabel2.Text = ex.Message;
			}
		}

		private void autoAvoidCheckBox_CheckedChanged(object sender, EventArgs e) {
			_strategy.setAutoAvoid(autoAvoidCheckBox.Checked);
		}

		private void manualOverrideCheckBox_CheckedChanged(object sender, EventArgs e) {
			_strategy.setManualOverride(manualOverrideCheckBox.Checked);
		}

		private void pIDTunerToolStripMenuItem_Click(object sender, EventArgs e) {
			PidTuner p = new PidTuner(ref _strategy);
			p.Show();
		}

		private void steeringSetpointSetButton_Click(object sender, EventArgs e) {
			try {
				_strategy.ManualSteering = int.Parse(steeringSetpointTextBox.Text);
				_strategy.steeringOut(_strategy.ManualSteering);
			} catch (Exception ex) {
				// post a non numeric message
				toolStripStatusLabel2.Text = ex.Message;
			}
		}

    }
}
