﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using RS232Port;

namespace PlatformController {
    public class PlatformData {

		public readonly double countsPerMeter;
		public readonly double throttleKp;
		public readonly double throttleKi;
		public readonly double throttleKd;
		public PlatformData() {
		}

		public PlatformData(double cntsPerMtr, double throtKp, double throtKi, double throtKd) {
			//pull out of ini file so this value can be tweaked/calibrated,
			//can change due to different wheels, spur/pinion gear, encoder, etc.
			countsPerMeter = cntsPerMtr;
			throttleKp = throtKp;
			throttleKi = throtKi;
			throttleKd = throtKd;

		}

        long _encoderCounts = 0;
        public long EncoderCounts {
            get { return _encoderCounts; }
        }

		public void setEncoderCounts(long count) {
			_encoderCounts = count;
		}

		public void incrementEncoder() {
			++_encoderCounts;
		}

		public void decrimentEncoder() {
			--_encoderCounts;
		}

		double _countsPerSecond = 0;
		public void setCountsPerSecond(double countsPerSecond) {
			_countsPerSecond = countsPerSecond;
		}
		public double CountsPerSecond {
			get { return _countsPerSecond; }
		}

		public void setCountsPerSecond(int counts) {
			_countsPerSecond = counts;
		}

		public double Meters {
			get { return (double)_encoderCounts / (double)countsPerMeter; }
		}

		public double Speed {
			get { return (double)_countsPerSecond / countsPerMeter; }
		}

		bool _autoReversing = false;
		public void setAutoReversing(bool autoReversing) {
			_autoReversing = autoReversing;
		}
		public bool AutoReversing {
			get { return _autoReversing; }
		}

        bool _reversing = false; //r:
		public void setReversing(bool reversing) {
			_reversing = reversing;
		}
        public bool Reversing {
            get { return _reversing; }
        }

		int _irInstruction = 7; //u:
		public void setIrInstruction(int irInstruction) {
			_irInstruction = irInstruction;
		}
		public int IrInstruction {
			get { return _irInstruction; }
		}

		//dtr on platform controller com port
        bool _manualOverride = false; //m:
		public void setManualOverride(bool manualOverride) {
			_manualOverride = manualOverride;
		}
        public bool ManualOverride {
            get { return _manualOverride; }
        }

		//rts on platform controller com port
		bool _autoAvoid = false; 
		public void setAutoAvoid(bool autoAvoid) {
			_autoAvoid = autoAvoid;
		}
		public bool AutoAvoid {
			get { return _autoAvoid; }
		}

        bool _leftBumper = false; //f:
		public void setLeftBumper(bool leftBumper) {
			_leftBumper = leftBumper;
		}
        public bool LeftBumper {
            get { return _leftBumper; }
        }

        bool _rightBumper = false; //g:
		public void setRightBumper(bool rightBumper) {
			_rightBumper = rightBumper;
		}
        public bool RightBumper {
            get { return _rightBumper; }
        }

        int _leftSonar = 0; //l:
		public void setLeftSonar(int leftSonar) {
			_leftSonar = leftSonar;
		}
		public double LeftSonarRaw {
			get { return _leftSonar; }
		}

        public double LeftSonar {
			get { return getSonarCm(_leftSonar); }
        }

		//the speed of sound is 29.034 microseconds per cm
		//must divide the counts by two to account for the return trip
		public double getSonarCm(int counts) {
			return ((counts * 8.68) / 2) / 29.034;
		}

        int _centerSonar = 0; //c:
		public void setCenterSonar(int centerSonar) {
			_centerSonar = centerSonar;
		}
		public double CenterSonarRaw {
			get { return _centerSonar; }
		}
        public double CenterSonar {
			get { return getSonarCm(_centerSonar); }
        }

        int _rightSonar = 0; //r:
		public void setRightSonar(int rightSonar) {
			_rightSonar = rightSonar;
		}
		public double RightSonarRaw {
			get { return _rightSonar; }
		}
        public double RightSonar {
			get { return getSonarCm(_rightSonar); }
        }

		int _throttleSetpoint = 0; //t:
		public int ThrottleSetpoint {
			get { return _throttleSetpoint; }
			set { _throttleSetpoint = value; }
		}

        int _throttle = 0; //t:
		public void setThrottle(int throttle) {
			_throttle = throttle;
		}
        public int Throttle {
            get { return _throttle; }
        }

        int _steering = 0; //s:
		public void setSteering(int steering) {
			_steering = steering;
		}
        public int Steering {
            get { return _steering; }
        }

        string _status = "not initialized";
		public void setStatus(string status) {
			_status = status;
		}
        public string Status {
            get { return _status; }
        }

        string _platformBuffer = "";
		public void setBuffer(string buffer) {
			_platformBuffer = buffer;
		}
        public string PlatformBuffer {
            get { return _platformBuffer; }
        }
    }
}
