﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Threading;
using System.Linq;
using System.Text;
using RS232Port;

namespace PlatformController {
	public class PlatformStrategy : SerialStrategy {

		Stopwatch countsPerSecondTimer;
		List<int> encoderCountList;
		RS232MainForm _encoderCounterPort;
		string _encoderCounterPortName = "COM3";
		string _javelinIdePortName = "COM1";

		int timeToAutoAvoid = 5000;
		double _countDelta = 0;
		public double CountDelta {
			get { return _countDelta; }
		}

		PlatformData _data;
		public PlatformData Data {
			get { return _data; }
			set { _data = value; }
		}

		int _manualThrottle = 0;
		public int ManualThrottle {
			get { return _manualThrottle; }
			set { _manualThrottle = value; }
		}

		int _manualSteering = 0;
		public int ManualSteering {
			get { return _manualSteering; }
			set { _manualSteering = value; }
		}

		Thread autoAvoidThread;

		public PlatformStrategy(string platformControllerPortName, string javelinIdePortName, string encoderCounterPortName
			, double countsPerMeter, double throttleKp, double throttleKi, double throttleKd) {
			serialPortName = platformControllerPortName;
			_javelinIdePortName = javelinIdePortName;
			_encoderCounterPortName = encoderCounterPortName;
			_data = new PlatformData(countsPerMeter, throttleKp, throttleKi, throttleKd);
			initializePlatformControllerPort();
			encoderCountList = new List<int>();
			PidControl.setPidCoefficients(_data.throttleKp, _data.throttleKi, _data.throttleKd);
			resetPlatformController();
			initializeEncoderCounterPort();
			countsPerSecondTimer = new Stopwatch();
			countsPerSecondTimer.Start();
			_watchdogTimer = new Stopwatch();
			_watchdogTimer.Start();

		}

		public void resetEncoderCounter() {
			countsPerSecondTimer.Reset();
			countsPerSecondTimer.Start();
			if (_encoderCounterPort.Port.IsOpen) {
				_encoderCounterPort.Port.DiscardInBuffer();
				_encoderCounterPort.Port.DtrEnable = true;
				Thread.Sleep(10);
				_encoderCounterPort.Port.DtrEnable = false;
			} else initializeEncoderCounterPort();
		}

		public void initializeEncoderCounterPort() {
			//open com 3 and toggle DTR 
			if (_encoderCounterPort == null) {
				_encoderCounterPort = new RS232MainForm();
				_encoderCounterPort.updateErrorEvent += updateError;
				_encoderCounterPort.DebugMode = RS232MainForm.debugTerminalMode.silent;
				_encoderCounterPort.Show();
				_encoderCounterPort.Hide();
				_encoderCounterPort.Port.PortName = _encoderCounterPortName;
				//format of encoder counter data is h;, where h=some hex number 
				//the minimum number of bytes in the buffer for 10 samples is 20 i.e. 0;0;...
				//as long as there are no atoms greaterthan 15 ther will never be more than 20 
				//bytes in the buffer per second. but 
				_encoderCounterPort.Port.ReceivedBytesThreshold = 10;
			}
			_encoderCounterPort.initializeComPort();
			if (_encoderCounterPort.Port.IsOpen) {
				_encoderCounterPort.Port.DtrEnable = true;
				Thread.Sleep(10);
				_encoderCounterPort.Port.DtrEnable = false;
			}
		}

		public void resetPlatformController() {
			//open com 1 and toggle DTR 
			RS232MainForm jide = new RS232MainForm();
			jide.updateErrorEvent += updateError;
			jide.DebugMode = RS232MainForm.debugTerminalMode.silent;
			jide.Show();
			jide.Hide();
			jide.Port.PortName = _javelinIdePortName;
			jide.initializeComPort();
			if (jide.Port.IsOpen) {
				jide.Port.DtrEnable = true;
				Thread.Sleep(10);
				jide.Port.DtrEnable = false;
				jide.closePort();
			}
		}

		public void initializePlatformControllerPort() {
			if (_rs232 == null) {
				_rs232 = new RS232MainForm();
				_rs232.updateErrorEvent += updateError;
				_rs232.Port.PortName = serialPortName;
				_rs232.Show();
				_rs232.Hide();
				_rs232.Text = "Platform Controller Serial Port";
				_rs232.DebugMode = RS232MainForm.debugTerminalMode.silent;
				_rs232.receiveDataEvent += receiveData;
				_rs232.ctsChangedEvent += ctsStateChanged;
				_rs232.dsrChangedEvent += dsrStateChanged;
				_rs232.setSilentModeEvent += updateSilentMode;
				_rs232.Port.ReceivedBytesThreshold = 1; //adjust this if the byte count from the platform controller changes
				_rs232.RecieveDelayMs = 60;
				_rs232.updatePortSettings();
				_status = openPort();
				setAutoAvoid(false);
				setManualOverride(false);
			}
		}

		//sets the steering value of the platform controller
		public void steeringOut(int value) {
			value += 50;
			if (value > 100) value = 100;
			else if (value < 0) value = 0;
			writeToPlatformController(PlatformCommands.cmdArray[(int)PlatformCommands
				.commands.setSteering], (char)Math.Abs(value));
		}

		//sets the throttle value of the platform controller
		private void throttleOut(int value) {
			value += 50;
			if (value > 100) value = 100;
			else if (value < 0) value = 0;
			writeToPlatformController(PlatformCommands.cmdArray[(int)PlatformCommands
				.commands.setThrottle], (char)Math.Abs(value));
		}

		private void throttleControl() {
			//excecute only if in auto mode and no reverse conditions or if in manual mode and no dead man
			if (_data.IrInstruction < 5 || _data.ManualOverride) {
				if (_manualThrottle > 50) _manualThrottle = 50;
				else if (_manualThrottle < -50) _manualThrottle = -50;
				if (_manualThrottle < 0 && _data.ThrottleSetpoint >= 0) _manualThrottle = 0;
				else if (_manualThrottle > 0 && _data.ThrottleSetpoint <= 0) _manualThrottle = 0;
				throttleOut(_manualThrottle);
			}
		}

		public void writeToPlatformController(PlatformCommand command, char argument) {
			_rs232.transmit(command.command.ToString() + argument.ToString());
		}

		private void receiveData() {
			if (_rs232.SerialData.Length > 0) {
				parsSerialData(_rs232.SerialData);
				updateUi();
				updateData();
				_watchdogTimer.Reset();
				_watchdogTimer.Start();
			}
		}

		double elapsedSeconds = 0;
		private void receiveEncoderData() {
			//if (countsPerSecondTimer.ElapsedMilliseconds != 0) {
			elapsedSeconds = (double)(countsPerSecondTimer.ElapsedMilliseconds) / 1000.0;
			countsPerSecondTimer.Reset();
			countsPerSecondTimer.Start();
			//}
			parsEncoderCounterData(_encoderCounterPort.SerialData);
			_data.setEncoderCounts(_data.EncoderCounts + (long)encoderCountList.Sum());
			_data.setCountsPerSecond((double)encoderCountList.Sum() / elapsedSeconds);
			if (!_data.AutoReversing && !_data.AutoAvoid && _data.IrInstruction != 7) {
				//calculate pid
				_manualThrottle += (int)Math.Round(PidControl
					.getMvFromInput(_data.ThrottleSetpoint, encoderCountList, elapsedSeconds), 0);
				//output to throttle
				throttleControl();
			} else _manualThrottle = 0;
		}

		private void ctsStateChanged() {
			//the cts input is connected to the reversing indicator 
			_data.setReversing(!_rs232.Port.CtsHolding);
		}

		private void dsrStateChanged() {
			//the dsr input is connected to the aout-reversing indicator  
			_data.setReversing(!_rs232.Port.DsrHolding);
			_data.setAutoReversing(!_rs232.Port.DsrHolding);
			if (!_data.AutoReversing && !_data.ManualOverride) {
				autoAvoidThread = new Thread(autoAvoid);
				autoAvoidThread.Start();
			}
		}

		public void setAutoAvoid(bool auto) {
			_rs232.Port.RtsEnable = !auto;
			_data.setAutoAvoid(auto);
		}

		public void setManualOverride(bool manual) {
			_rs232.Port.DtrEnable = manual;
			_data.setManualOverride(manual);
		}

		public void autoAvoid() {
			setAutoAvoid(true);
			Thread.Sleep(timeToAutoAvoid);
			setAutoAvoid(false);
		}


		public void parsEncoderCounterData(string buffer) {
			encoderCountList.Clear();
			string s = "";
			try {
				if (buffer[0] == ';') buffer = buffer.Remove(0, 1);
				while (buffer.Contains(";")) {
					s = buffer.Substring(0, buffer.IndexOf(";"));
					buffer = buffer.Remove(0, buffer.IndexOf(";") + 1);
					if (_data.Reversing) encoderCountList.Add(int.Parse
						(s, System.Globalization.NumberStyles.HexNumber) * -1);
					else encoderCountList.Add(int.Parse(s, System.Globalization
						.NumberStyles.HexNumber));
				}
			} catch (Exception e) {
				updateError(e);
				_error = e.Message + "\r\n" + e.StackTrace;
			}
		}

		//part of the strategy class is the parsing algorithm 
		//for the platform controller's serial data.
		public void parsSerialData(string buffer) {
			try {
				_data.setBuffer(buffer);
				string s = "";
				char c = '0';
				if (buffer[1] != ':')
					buffer = buffer.Remove(0, buffer.IndexOf(":", 1) - 1);
				while (buffer.Contains(":") && buffer.Contains(";")) {
					c = buffer[0];
					switch (c) {
						case '0':
							s = buffer.Substring(2, 1);
							buffer = buffer.Remove(0, 4);
							_data.setStatus(((byte)s[0]).ToString());
							updateStatus();
							_rs232.Port.DiscardInBuffer();
							_encoderCounterPort
								.receiveDataEvent += receiveEncoderData;
							break;
						case 'a':
							s = buffer.Substring(2, 1);
							buffer = buffer.Remove(0, 4);
							byte[] b = { (byte)s[0] };
							_dataAtom = new BitArray(b);
							_data.setManualOverride(_dataAtom[2]);
							_data.setLeftBumper(_dataAtom[1]);
							_data.setRightBumper(_dataAtom[0]);
							break;
						case 'b':
							s = buffer.Substring(2, 2);
							buffer = buffer.Remove(0, 5);
							_data.setLeftSonar(((byte)s[0] << 8) + (byte)s[1]);
							break;
						case 'c':
							s = buffer.Substring(2, 2);
							buffer = buffer.Remove(0, 5);
							_data.setCenterSonar(((byte)s[0] << 8) + (byte)s[1]);
							break;
						case 'd':
							s = buffer.Substring(2, 2);
							buffer = buffer.Remove(0, 5);
							_data.setRightSonar(((byte)s[0] << 8) + (byte)s[1]);
							break;
						case 'e':
							s = buffer.Substring(2, 1);
							buffer = buffer.Remove(0, 4);
							_data.setSteering((byte)s[0] - 50);
							break;
						case 'f':
							s = buffer.Substring(2, 1);
							buffer = buffer.Remove(0, 4);
							_data.setThrottle((byte)s[0] - 50);
							break;
						case 'g':
							s = buffer.Substring(2, 1);
							buffer = buffer.Remove(0, 4);
							_data.setIrInstruction((byte)s[0]);
							break;
						default:
							_rs232.Port.DiscardInBuffer();
							break;
					}
				}
			} catch (Exception e) {
				updateError(e);
				_error = e.Message + "\r\n" + e.StackTrace;
			}
		}
	}
}
