﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Linq;
using System.Text;

namespace RS232Port {
	public class SerialStrategy {

		public delegate void updateStatusEventHandler();
		public event updateStatusEventHandler updateStatusEvent;
		public event updateStatusEventHandler setSilentModeEvent;
		public delegate void updateDataEventHandler();
		public event updateDataEventHandler updateDataEvent;
		public delegate void updateUiEventHandler();
		public event updateUiEventHandler updateUiEvent;
		public delegate void updateErrorEventHandler(Exception error);
		public event updateErrorEventHandler updateErrorEvent;
		protected delegate void openSerialPortEventHandler();
		protected event openSerialPortEventHandler openSerialPortEvent;

		protected BitArray _dataAtom;
		protected double _outDouble = 0;

		protected string serialPortName;
		protected RS232MainForm _rs232;

		protected string _status;
		public string Status {
			get { return _status; }
		}

		protected string _error;
		public string Error {
			get { return _error; }
		}

		protected Stopwatch _watchdogTimer;
		public Stopwatch WatchdogTimer {
			get { return _watchdogTimer; }
		}

		public SerialStrategy() {
			_watchdogTimer = new Stopwatch();
		}

		protected void updateData() {
			if (updateDataEvent != null)
				updateDataEvent.Invoke();
		}

		protected void updateStatus() {
			if (updateStatusEvent != null)
				updateStatusEvent.Invoke();
		}

		protected void updateSilentMode() {
			_silent = _rs232.Silent;
			if (setSilentModeEvent != null)
				setSilentModeEvent.Invoke();
		}
		protected void updateUi() {
			if (updateUiEvent != null)
				updateUiEvent.Invoke();
		}

		//called by the rs232 port if there is an error
		//chain of responsibility
		protected void updateError(Exception e) {
			if (updateErrorEvent != null)
				updateErrorEvent.Invoke(e);
		}

		public string openPort() {
			return _rs232.initializeComPort();
		}

		public string closePort() {
			return _rs232.closePort();
		}

		public void setSilentMode(bool silent) {
			_rs232.Silent = silent;
		}

		protected bool _silent = true;
		public bool Silent {
			get { return _silent; }
		}

		public void showSerialPort() {
			if (!_rs232.IsDisposed)
				_rs232.Show();
			else {
				if (openSerialPortEvent != null)
					openSerialPortEvent.Invoke();
				_rs232.Show();
			}
		}

	}
}
