using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace VisionSystem {
    public partial class VisionSystemMainForm : Form {
        public delegate void updateUiEventHandler();
        public event updateUiEventHandler updateUiEvent;
		public delegate void updateDataEventHandler();
		public event updateDataEventHandler updateDataEvent;
		public delegate void updateErrorEventHandler(Exception error);
		public event updateErrorEventHandler updateErrorEvent;
		public delegate void cameraEventHandler();
		public event cameraEventHandler stopCameraEvent;
		public event cameraEventHandler startCameraEvent;

		VisionData _data;
		public VisionData Data {
			get { return _data; }
		}

		string _status = "";
		public string Status {
			get { return _status; }
		}

		bool _silent = true;
		public bool Silent {
			get { return _silent; }
			set {
				_silent = value;
				if (_silent)
					_data.updateUiEvent -= onConeAquired;
				else
					_data.updateUiEvent += onConeAquired;
			}
		}

		Bitmap _filtered;

        public VisionSystemMainForm() {
            InitializeComponent();
			initializeObject(224, 48, 42);
        }

		public VisionSystemMainForm(int red, int green, int blue) {
			InitializeComponent();
			initializeObject(red, green, blue);
		}

		private void initializeObject(int red, int green, int blue) {
			_filtered = new Bitmap((int)cameraDsCapture.VideoSize.Width
				, (int)cameraDsCapture.VideoSize.Height);
			_data = new VisionData(ref _filtered);
			updateUiEvent += onConeAquired;
			startCameraEvent += startCamera;
			stopCameraEvent += stopCamera;
			rTextBox.Text = _data.Orange.R.ToString();
			gTextBox.Text = _data.Orange.G.ToString();
			bTextBox.Text = _data.Orange.B.ToString();
			rTTextBox.Text = _data.RedT.ToString();
			gTTextBox.Text = _data.GrnT.ToString();
			bTTextBox.Text = _data.BluT.ToString();
			setColor(Color.FromArgb(red, green, blue));
		}

		private void onConeAquired() {
			if (coneAquiredTextBox.InvokeRequired) {
				try {
					this.Invoke(updateUiEvent);
				} catch (Exception e) {
					if(updateErrorEvent != null)
						updateErrorEvent.Invoke(e);
					_status = e.Message;
				}
			} else {
				try {
					filteredConePictureBox.Image = _filtered;
					coneAquiredTextBox.Text = _data.ConeAquired.ToString();
					turnMagnitudeTextBox.Text = _data.TurnMagnitude.ToString();
					turnDierectionTextBox.Text = _data.TurnDirection;
					deltaTextBox.Text = _data.DeltaX.ToString();
					leftSlopeTextBox.Text = Math.Round(_data.LeftSlope, 2).ToString();
					rightSlopeTextBox.Text = Math.Round(_data.RightSlope, 2).ToString();
					scaleFactorTextBox.Text = _data.ScaleFactor.ToString();
				} catch (Exception e) {
					if (updateErrorEvent != null)
						updateErrorEvent.Invoke(e);
					_status = e.Message;
				}
			}
		}

		private void coneGenericFilter_ProcessData(object Sender
			, Mitov.VideoLab.VideoProcessDataEventArgs Args) {
			_data.processData(Args.InBuffer);
			unfilteredConePictureBox.Image = Args.InBuffer.ToBitmap();
			if (updateDataEvent != null) {
				updateDataEvent.Invoke();
			}
		}

        private void saveOriginal_Click(object sender, EventArgs e) {
            saveRawImage();
        }

        private void saveFilter_Click(object sender, EventArgs e) {
            savefilteredImage();
        }

		public void saveRawImage() {
			try {
				string dt = DateTime.Now.ToString().Replace('/', '-');
				dt = dt.Replace(':', '-');
				toolStripStatusLabel2.Text = "saving raw image as a .bmp";
				unfilteredConePictureBox.Image.Save(".\\images\\" + dt + "_original.bmp");
				toolStripStatusLabel2.Text = "raw image saved";
			} catch (Exception e) {
				if (updateErrorEvent != null)
					updateErrorEvent.Invoke(e);
			}
		}

        public void savefilteredImage() {
			try {
				string dt = DateTime.Now.ToString().Replace('/', '-');
				dt = dt.Replace(':', '-');
				toolStripStatusLabel2.Text = "saving filter image as a .bmp";
				_filtered.Save(".\\images\\" + dt + "_filter.bmp");
				toolStripStatusLabel2.Text = "filter image Saved";
			} catch (Exception e) {
				if (updateErrorEvent != null)
					updateErrorEvent.Invoke(e);
			}
        }

        private void editFilterButton_Click(object sender, EventArgs e) {
            setFilter();
        }

        private void setFilter() {
            try {
                _data.setFilter(Byte.Parse(rTTextBox.Text)
                    , Byte.Parse(gTTextBox.Text)
                    , Byte.Parse(bTTextBox.Text));
                toolStripStatusLabel2.Text = "changed cilter";
            } catch (Exception e) {
				if (updateErrorEvent != null)
					updateErrorEvent.Invoke(e);
                toolStripStatusLabel2.BackColor = Color.Red;
                toolStripStatusLabel2.Text
                    = "non-numeric value entered, or entry out of range";
            }
        }

        private void pickColorButton_Click(object sender, EventArgs e) {
            colorDialog1.ShowDialog();
            setColor(colorDialog1.Color);
        }

        private void videoDeviceButton_Click(object sender, EventArgs e) {
            cameraDsCapture.Stop();
            cameraDsCapture.VideoCaptureDevice.ShowDeviceSelctDialog();
            toolStripStatusLabel2.Text = "video device selected";
        }

        public void initializeCamera() {
			cameraDsCapture.Stop();
			cameraDsCapture.VideoCaptureDevice.DeviceName = "USB Video Device";
			//toolStripStatusLabel2.Text = "video device selected";
        }

        private void startDeviceButton_Click(object sender, EventArgs e) {
            startCamera();
        }

		public void startCamera() {
			if (silentCheckBox.InvokeRequired) {
				this.Invoke(startCameraEvent);
			} else {
				if (!cameraDsCapture.Opened) {
					toolStripStatusLabel2.Text = "starting video device";
					//starting the device automatically opens it
					//but stoping it does not automatically close it.
					cameraDsCapture.Start();
					toolStripStatusLabel2.Text = "video device started";
					toolStripStatusLabel1.BackColor = Color.Green;
					toolStripStatusLabel1.Text = "vis_init";
				}
			}
		}

       public void stopDeviceButton_Click(object sender, EventArgs e) {
            stopCamera();
        }

	   public void stopCamera() {
		   if (silentCheckBox.InvokeRequired) {
			   this.Invoke(stopCameraEvent);
		   } else {
			   if (cameraDsCapture.Opened) {
				   silentCheckBox.Checked = true;
				   cameraDsCapture.Stop();
				   cameraDsCapture.Close();
				   toolStripStatusLabel2.Text = "video device stopped";
				   //invoke a delegate assigned to this method
				   toolStripStatusLabel1.BackColor = Color.Red;
				   toolStripStatusLabel1.Text = "not initialized";
			   }
		   }
	   }

        private void setColor(Color c) {
            _data.setColor(c);
            rTextBox.Text = _data.Orange.R.ToString();
            gTextBox.Text = _data.Orange.G.ToString();
            bTextBox.Text = _data.Orange.B.ToString();
            toolStripStatusLabel2.Text = "color changed";
        }

        private void hideButton_Click(object sender, EventArgs e) {
            this.Hide();
        }

        private void silentCheckBox_CheckedChanged(object sender, EventArgs e) {
            Silent = silentCheckBox.Checked;	
        }

        private void unfilteredConePictureBox_MouseDoubleClick
            (object sender, MouseEventArgs e) {
            setColor(((Bitmap)(unfilteredConePictureBox.Image)).GetPixel(e.X, e.Y));
        }

		private void VisionSystemMainForm_FormClosing(object sender, FormClosingEventArgs e) {
			stopCamera();
		}

    }
}